#ifndef __command_h__
#define __command_h__

#include "tokens.h"

/*
 * Aufzaehlung aller erlaubten Adressmodes
 */
enum AdrMode {
  am_none = 0, am_direct = 1, am_regind = 2, am_ripostinc = 3,
  am_ripredec = 4, am_absolut = 5, am_pcrel = 6, am_immed = 7,
  am_riadrdist = 8
};


/*
 * Genaue Definition eines Argumentes eines Befehles
 */
struct ArgAdrMode {
  enum AdrMode	  aam_mode;
  char          * aam_ID;	/* Register oder Label */
  int             aam_hex;	/* Adresse usw. */
};



/*
 * Beispiel fuer eine Copdierung der Addressmodes.
 * Muss je nach Prozessor natuerlich angepasst werden.
 */
char * AdrModesCodes[] =
{ "xxx", "000", "001", "010", "011", "100", "101", "110", "111" };


#define MAXARGS		3	/* soviele Argumente darf ein Bef. maximal haben */

/*
 * Array mit dem festgelegt werden kann, welche Adressmodi
 * fuer ein Argument erlaubt sind und welche nicht.
 */
struct ValidAdrModes {
  int  am_RegDirect;
  int  am_RegIndirect;
  int  am_RegIndirectPostIncr;
  int  am_RegIndirectPreDecr;
  int  am_Absolut;
  int  am_PCrel;
  int  am_Immediate;
  int  am_RegIndirectAdrDist;
};


/*
 * Command Struktur
 *
 * Hier werden alle Maschinenbefehle genau spezifiziert.
 * Wichtig fuer das Programm sind:
 *  co_BefLength : Wie lang (in 16 Bit Worten) ist der Machinenbefehl
 *  co_NumArgs   : Wieviele Argumente hat der Befehl
 *  co_AdrModesArg : Welche Adressierungsarten sind fuer die
 *			einzelnen Argumente erlaubt.
 *
 */
struct Command {
  char                * co_name;
  char                * co_opcode;		  /* Z.B.: "00111100xxxxyyyy" */
  int                   co_BefLength;	          /* in 16-Bit Worten */
  int		        co_NumArgs;		  /* Anzahl der Argumente (hier 0-MAXARGS) */
  struct ValidAdrModes  co_AdrModesArg[MAXARGS];  /* gueltige Adr. Modes fuer Arg */
};

struct Command ComTab[] =
 {
	{ "(nil)",	NULL,   0, 0, { { 0,0,0,0,0,0,0,0 }, { 0,0,0,0,0,0,0,0 }, { 0,0,0,0,0,0,0,0 } } },
	{ "addq",	"0011", 1, 2, { { 0,0,0,0,0,0,1,0 }, { 1,1,1,1,1,0,0,1 }, { 0,0,0,0,0,0,0,0 } } },
	{ "add",	"1000", 3, 2, { { 1,1,1,1,1,1,1,1 }, { 1,1,1,1,1,1,0,1 }, { 0,0,0,0,0,0,0,0 } } },
	{ "and",	"1100", 2, 2, { { 1,1,1,1,1,1,1,1 }, { 1,0,0,0,0,0,0,0 }, { 0,0,0,0,0,0,0,0 } } },
	{ "clr",	"1110", 1, 1, { { 1,1,1,1,1,0,0,1 }, { 0,0,0,0,0,0,0,0 }, { 0,0,0,0,0,0,0,0 } } },
	{ "cmp",	"1111", 2, 2, { { 1,1,1,1,1,1,1,1 }, { 1,0,0,0,0,0,0,0 }, { 0,0,0,0,0,0,0,0 } } },
	{ "jmp",	"0101", 2, 1, { { 0,1,0,0,1,1,0,1 }, { 0,0,0,0,0,0,0,0 }, { 0,0,0,0,0,0,0,0 } } },
	{ "jsr",	"1101", 2, 1, { { 0,1,0,0,1,1,0,1 }, { 0,0,0,0,0,0,0,0 }, { 0,0,0,0,0,0,0,0 } } },
	{ "lea",	"0001", 2, 2, { { 0,1,0,0,1,1,0,0 }, { 1,0,0,0,0,0,0,0 }, { 0,0,0,0,0,0,0,0 } } },
	{ "move",	"0111", 3, 2, { { 1,1,1,1,1,1,1,1 }, { 1,1,1,1,1,0,0,1 }, { 0,0,0,0,0,0,0,0 } } },
	{ "nop",	"0000", 1, 0, { { 0,0,0,0,0,0,0,0 }, { 0,0,0,0,0,0,0,0 }, { 0,0,0,0,0,0,0,0 } } }
 };

#define NUMCOMMANDS	(sizeof(ComTab) / sizeof(struct Command))

#endif
