
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>

#include "symtab.h"


#define SYMTABSIZE	211	/* muss eine Primzahl sein */

struct SymTabEntry {
  char *                ste_name;
  int  	                ste_data;	/* was auch immer */
  struct SymTabEntry  * ste_next;
};
 
static struct SymTabEntry * SymTab[SYMTABSIZE];



static inline int HashVal(char * s);
static struct SymTabEntry * get_ele(char * name);
static void insert_ele(struct SymTabEntry * newele);




/*
 * Traegt einen Eintrag neu ein.
 * Liefert 0 zurueck, wenn alles ok,
 * -1 sonst (z.B. Wenn es einen Eintrag dieses Namens schon gab).
 */
int InsertEntry(char * name, int data)
{
  struct SymTabEntry * newele;
  
  if (get_ele(name)) return -1;
  newele = calloc(1, sizeof(struct SymTabEntry));
  assert(newele);

  newele->ste_name = strdup(name);
  newele->ste_data = data;

  insert_ele(newele);
  return 0;
}

/*
 * Testet, ob es einen Eintrag mit diesem Namen schon gibt
 */
int ExistEntry(char * name)
{
  return get_ele(name) != NULL;
}

/*
 * Liefert in data die Daten des Eintrags mit dem Namen zurueck.
 * Return: 0 wenn alles ok, -1 sonst (z.B. Eintrag existiert nicht)
 */
int GetEntryData(char * name, int * data)
{
  struct SymTabEntry * ele = get_ele(name);
  assert(data);
  if (ele) {
    *data = ele->ste_data;
    return 0;
  }
  else return -1;
}

/*
 * Aendert das Datum des angegebenen Eintrages.
 * Return: 0 wenn alles ok, -1 sonst (z.B. Eintrag existiert nicht)
 */
int ChangeData(char * name, int newdata)
{
  struct SymTabEntry * ele = get_ele(name);
  if (ele) {
    ele->ste_data = newdata;
    return 0;
  }
  else return -1;
}



/*--------------------------------------------------------*/

/*
 * nun folgen interne Hilfs-Funktionen
 */



/*
 * unsigned int hashpjw(const char *)
 *
 * Berechnet eine Hash-Zahl zu dem uebergebenen String.
 * Der Wertebereich muss noch selbst eingeschraenkt werden.
 *
 * Die Funktion wurde dem Drachenbuch [Aho, Sethi, Ullman]
 * (deutsche Version, Band 1, Seite 535) entnommen.
 *
 * Bei GNU G++ existiert diese Funktion bereits. Sie wird in
 * dem Includefile 'builtin.h' deklariert.
 *
 */


static unsigned int hashpjw(const char * s)
{
  unsigned int h = 0;
  unsigned int g;
  
  while (*s != 0) {
    h = (h << 4) + *s++;
    if ((g = h & 0xf0000000) != 0) {
      h = (h ^ (g >> 24)) ^ g;
    }
  }
  return h;
}


static inline int HashVal(char * s)
{
  return hashpjw(s) % SYMTABSIZE;
}


static struct SymTabEntry * get_ele(char * name)
{
  struct SymTabEntry * ele = SymTab[HashVal(name)];
  while (ele && strcmp(name, ele->ste_name)) ele = ele->ste_next;
  return ele;
}


static void insert_ele(struct SymTabEntry * newele)
{
  int hash = HashVal(newele->ste_name);
  newele->ste_next = SymTab[hash];
  SymTab[hash] = newele;
}




